/* globals.c */

/* VIS-5D version 3.0 */

/*
vis5d program for visualizing five dimensional gridded data sets
Copyright (C) 1990, 1991, 1992  Bill Hibbard, Brian Paul, Dave Santek,
and Andre Battaiola.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include "globals.h"
#include "sync.h"



int Quiet, Debug;

char ProgramName[128];

char DataFile[100];


/*** Data Set / Grid stuff ***/
int Nr, Nc, Nl;
int NumTimes=1;
int NumParms;
int TimeStamp[MAXTIMES];
int DayStamp[MAXTIMES];
char ParmName[MAXPARMS][10];

float MinVal[MAXPARMS];   /* Min value for each parameter */
float MaxVal[MAXPARMS];   /* Max value for each parameter */

int Elapsed[MAXTIMES];    /* time in seconds relative to first time step */

/* Geographic location for each time step */
float NorthLat[MAXTIMES], SouthLat[MAXTIMES];
float WestLon[MAXTIMES], EastLon[MAXTIMES];
float TopHgt[MAXTIMES], BottomHgt[MAXTIMES];

float LatInc, LonInc, HgtInc; /* Geographic resolution per grid point */

float NorthBound, SouthBound; /* max of NorthLat's, min of SouthLat's */
float WestBound, EastBound;   /* max of WestLon's, min of EastLon's */
float TopBound, BottomBound;  /* max of TopHgt's, min of BottomHgt's */

int StaticDomain;    /* True if all NorthLat's, SouthLat's, etc are equal */

int Uparm, Vparm, Wparm;
int WindAvailable;   /* U,V,W wind info available? */

int NumClones=0;            /* Number of cloned variables */
int CloneTable[MAXPARMS];   /* Clone to real variable translation table */


/* Most recently requested graphics values: */
float IsoLevel[MAXPARMS];        /* in [MinVal..MaxVal] */
float HSliceInterval[MAXPARMS];
float HSliceLevel[MAXPARMS];     /* in [0..Nl-1] */
float VSliceInterval[MAXPARMS];
float VSliceR1[MAXPARMS];        /* in [0..Nr-1] */
float VSliceC1[MAXPARMS];        /* in [0..Nc-1] */
float VSliceR2[MAXPARMS];        /* in [0..Nr-1] */
float VSliceC2[MAXPARMS];        /* in [0..Nc-1] */
float CHSliceLevel[MAXPARMS];    /* in [0..Nl-1] */
uint  CHSliceColors[MAXPARMS][256]; /* data value --> color LUT */
float CVSliceR1[MAXPARMS];       /* in [0..Nr-1] */
float CVSliceC1[MAXPARMS];       /* in [0..Nc-1] */
float CVSliceR2[MAXPARMS];       /* in [0..Nr-1] */
float CVSliceC2[MAXPARMS];       /* in [0..Nc-1] */
uint  CVSliceColors[MAXPARMS][256]; /* data value --> color LUT */
float HWindLevel[WINDSLICES];    /* in [0..Nl-1] */
float HWindDensity[WINDSLICES];  /* around 1.0 */
float HWindScale[WINDSLICES];    /* around 1.0 */
float VWindR1[WINDSLICES];       /* in [0..Nr-1] */
float VWindC1[WINDSLICES];       /* in [0..Nr-1] */
float VWindR2[WINDSLICES];       /* in [0..Nr-1] */
float VWindC2[WINDSLICES];       /* in [0..Nr-1] */
float VWindDensity[WINDSLICES];  /* around 1.0 */
float VWindScale[WINDSLICES];    /* around 1.0 */

float TrajStep = 100.0;
float TrajLength = 1.0;
int RibbonFlag = 0;


/*** User Interface ***/
int CurTime=0;   /* the currently displayed time step in [0..NumTimes-1] */
int GoTime=0;    /* animate flag (0=off, -1=backward, 1=forward) */

/* graphics display flags */
int DisplaySurf[MAXPARMS];
int DisplayHSlice[MAXPARMS];
int DisplayVSlice[MAXPARMS];
int DisplayCHSlice[MAXPARMS];
int DisplayCVSlice[MAXPARMS];
int DisplayHWind[WINDSLICES];
int DisplayVWind[WINDSLICES];
int DisplayTraj[TRAJGROUPS];

int ContnumFlag=1;    /* display contour numbers flag */
int TopoFlag;         /* is topography available? */
int DisplayTopo;      /* display topography? */
int MapFlag;          /* is map available? */
int DisplayMap;       /* display map? */
int CoordFlag;        /* Geographic (0) or grid(1) coordinate display */
int DisplayBox=1;     /* Display 3-D box */
int DisplayClock=1;   /* Display clock */
int PrettyFlag;       /* 'Pretty' rendering mode */
int DisplayInfo;      /* Display information flag */

int Redraw;           /* set when we want to redraw window */
int AnimRate;         /* minimum time between animation steps in msec */

int DisplayCursor;


/*** Graphics / Rendering ***/
Matrix  CTM = { DEFAULT_SCALE, 0.0, 0.0, 0.0,
		0.0, DEFAULT_SCALE, 0.0, 0.0, 
		0.0, 0.0, DEFAULT_SCALE, 0.0,
		0.0, 0.0, 0.0, 1.0 };
Matrix  CTMinv;


/* Size of box in graphics coords */
float Xmin, Xmax;  /* West, East */
float Ymin, Ymax;  /* South, North */
float Zmin, Zmax;  /* Top, Bottom */

float Gxscale, Gyscale, Gzscale;


float CursorX, CursorY, CursorZ;     /* cursor position in graphics coords */
float CursorR, CursorC, CursorL;     /* in grid coords */


/* Colors */
unsigned long Color[MAXPARMS][5] =
   {0xffffff00, 0xffffffff, 0xffff00ff, 0xff00ffff, 0xffff0000,
    0xff00ff00, 0xff0000ff, 0xff8080ff, 0xff80ff80, 0xffff8080,
    0xffff8000, 0xffff0080, 0xff80ff00, 0xff00ff80, 0xff8000ff,
    0xff0080ff, 0xffc0c040, 0xffc040c0, 0xff40c0c0, 0xff4080ff,
    0xff8080ff, 0xffc0ffc0, 0xff40c0ff, 0xff04ffc0, 0xffff40c0
};
unsigned long HWindColor[WINDSLICES] = { 0xff00ffff, 0xff00ff00 };
unsigned long VWindColor[WINDSLICES] = { 0xff0088ff, 0xffff00ff };
unsigned long TrajColor[TRAJGROUPS] = {
    0xffffffff, 0xffffff00, 0xffff00ff, 0xff00ff00,
    0xff0088ff, 0xffff0088, 0xffff8800, 0xff008888
};
unsigned long *CursorColor = TrajColor;
unsigned long BoxColor = 0xffffffff;
unsigned long BgColor = 0xff000000;
unsigned long LabelColor = 0xffffffff;


/*** Graphics Data Structures ***/

LOCK Gfx_Lock;   /* to synchronize reads/writes to graphics below */

struct surface  SurfTable[MAXPARMS][MAXTIMES];
struct hslice   HSliceTable[MAXPARMS][MAXTIMES];
struct vslice   VSliceTable[MAXPARMS][MAXTIMES];
struct chslice  CHSliceTable[MAXPARMS][MAXTIMES];
struct cvslice  CVSliceTable[MAXPARMS][MAXTIMES];
struct hwind    HWindTable[WINDSLICES][MAXTIMES];
struct vwind    VWindTable[WINDSLICES][MAXTIMES];
struct traj     TrajTable[MAXTRAJ];
int NumTraj;

struct label *FirstLabel;

